#define KB_BELLS_IOCTL_BASE 0xbb
#define BELL_SOUND 1
#define DUPE_SOUND 2
#define MESSAGE_SOUND 3
#define BAND_CHANGE_SOUND 4
#define SINGLE_MULT_SOUND 5
#define DOUBLE_MULT_SOUND 6
#define BAD_CALL_SOUND 7
#define BAD_KEY_SOUND 8
#define SPEED_CHANGE_SOUND 9
